@@ -2,12 +2,16 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
+from contextlib import contextmanager, redirect_stderr
 import importlib.util
+from io import StringIO
 import os
 import pathlib
 import sys
 import tempfile
 import unittest
+from unittest.mock import patch
+import warnings
 
 from oxidized_importer import (
     OxidizedFinder,
@@ -17,6 +21,24 @@ from oxidized_importer import (
 )
 
 
+@contextmanager
+def assert_tempfile_cleaned_up(TemporaryDirectory=tempfile.TemporaryDirectory):
+    call_count = 0
+    class TrackingTempDir(TemporaryDirectory):
+        def cleanup(self):
+            nonlocal call_count
+            call_count += 1
+            super().cleanup()
+
+    patcher = patch("tempfile.TemporaryDirectory", TrackingTempDir)
+    msg = fr"""^Implicitly cleaning up <{TrackingTempDir.__name__} (['"]).*\1>$"""
+    with warnings.catch_warnings(record=True) as cm, patcher:
+        warnings.filterwarnings(
+            "error", category=ResourceWarning, module=r"^tempfile$", message=msg)
+        yield TrackingTempDir, TrackingTempDir.cleanup
+    assert call_count == 1, f"tempfile.TemporaryDirectory.cleanup {call_count=}≠1"
+
+
 class TestImporterResourceCollector(unittest.TestCase):
     def setUp(self):
         self.raw_temp_dir = tempfile.TemporaryDirectory(
@@ -36,6 +58,23 @@ class TestImporterResourceCollector(unittest.TestCase):
         c = OxidizedResourceCollector(allowed_locations=["in-memory"])
         self.assertEqual(c.allowed_locations, ["in-memory"])
 
+    def test_tempdir_error(self):
+        class BadTempDir(tempfile.TemporaryDirectory):
+            def cleanup(self):
+                super().cleanup()
+                raise FileNotFoundError(self.name)
+        python_exe = os.environ.get("PYTHON_SYS_EXECUTABLE")
+        c = OxidizedResourceCollector(allowed_locations=["in-memory"])
+        stderr = StringIO()
+        assertion = assert_tempfile_cleaned_up(BadTempDir)
+        with assertion as (TrackingTempDir, cleanup), redirect_stderr(stderr):
+            oxide = c.oxidize(python_exe=python_exe)
+        self.assertRegex(
+            stderr.getvalue(),
+            fr"""Exception ignored in: <bound method {cleanup.__qualname__} """
+            fr"""of <{TrackingTempDir.__name__} (['"]){tempfile.gettempdir()}"""
+            fr"""[/\\].+\1>>""")
+
     def test_source_module(self):
         c = OxidizedResourceCollector(allowed_locations=["in-memory"])
 
@@ -48,7 +87,10 @@ class TestImporterResourceCollector(unittest.TestCase):
             c.add_in_memory(resource)
 
         f = OxidizedFinder()
-        f.add_resources(c.oxidize()[0])
+        python_exe = os.environ.get("PYTHON_SYS_EXECUTABLE")
+        with assert_tempfile_cleaned_up():
+            oxide = c.oxidize(python_exe=python_exe)
+        f.add_resources(oxide[0])
 
         resources = [r for r in f.indexed_resources() if r.name == "foo"]
         self.assertEqual(len(resources), 1)
@@ -67,14 +109,17 @@ class TestImporterResourceCollector(unittest.TestCase):
                     c.add_in_memory(resource)
                     c.add_filesystem_relative("", resource)
 
-        resources, file_installs = c.oxidize()
+        python_exe = os.environ.get("PYTHON_SYS_EXECUTABLE")
+        with assert_tempfile_cleaned_up():
+            resources, file_installs = c.oxidize(python_exe=python_exe)
         f = OxidizedFinder()
         f.add_resources(resources)
 
         with (self.td / "serialized").open("wb") as fh:
             fh.write(f.serialize_indexed_resources())
 
-        f = OxidizedFinder(resources_file=self.td / "serialized")
+        f = OxidizedFinder()
+        f.index_file_memory_mapped(self.td / "serialized")
 
         self.assertGreaterEqual(len(f.indexed_resources()), len(resources))
 
@@ -93,7 +138,9 @@ class TestImporterResourceCollector(unittest.TestCase):
                             if resource.optimize_level == 0:
                                 c.add_filesystem_relative("lib", resource)
 
-        resources, file_installs = c.oxidize()
+        python_exe = os.environ.get("PYTHON_SYS_EXECUTABLE")
+        with assert_tempfile_cleaned_up():
+            resources, file_installs = c.oxidize(python_exe=python_exe)
         self.assertEqual(len(resources), len(file_installs))
 
         idx = None
@@ -117,4 +164,4 @@ class TestImporterResourceCollector(unittest.TestCase):
 
 
 if __name__ == "__main__":
-    unittest.main(exit=False)
+    unittest.main()
